/*
   _===_ _           _                 ______ _     _             _ _ _
  |  ___| |         | |                | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ _____| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \_____| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |    | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/     \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|


  Contrôle du DDS AD9850 via STM32 Mini
  Électro-Bidouilleur, Octobre 2016
*/  

//------------------------------------------------------------------------------
 // Définitions au compilateur
 
 // Broches vers le AD9850
 #define DATA PB6        // Broche de données série (DATA)
 #define W_CLK PB7       // Broche d'horloge (CLK)
 #define FQ_UD PB8       // Broche FQ_UD
 #define RESET PB9       // Broche reset (RST).
 
 // Autres définitions
 #define pinLED PC13    // Broche de la DEL sur la plaquette STM32 Mini
 #define LF 10           // line feed (ASCII-10)
 
 // Routine d'envoi d'un bit
 #define pulseHigh(pin) {digitalWrite(pin, HIGH); delay(1); digitalWrite(pin, LOW); delay(1);}


//------------------------------------------------------------------------------
 // Routine de Transfert série d'un Octet vers le AD9850, un bit à la fois, le bit inférieur en premier

void xfert_byte(byte data)
{
  for (int i=0; i<8; i++, data>>=1)  // boucle d'envoi des 8 bits
  {
    digitalWrite(DATA, data & 0x01);    // envoi d'un bit
    delay(1);                           // délai de 1ms
    pulseHigh(W_CLK);   //apres chaque bit, un coup d'horloge
  }
}

//------------------------------------------------------------------------------
 // Envoi du mot de 40 bits représentant la fréquence, les bits de contrôle et la phase du AD9850

void sendFrequency(int freq) 
{
  for (int b=0; b<4; b++, freq>>=8) // Envoi des 40 bits, un octet à la fois
  {
    xfert_byte(freq & 0xFF); // envoi d'un octet
  }
  xfert_byte(0x000);   // envoi du dernier octet (contrôle et phase), tous des zéros
  pulseHigh(FQ_UD);  // Saisie de la nouvelle fréquence dans le AD9850. La nouvelle fréquence devrait sortir
}

//------------------------------------------------------------------------------
 // fonction de vérification de nombre. Retourne Vrai si la chaîne de caractères représente un nombre

boolean IsNumeric(String str) 
{
  if (str.length() == 0)  return false;   // si la chaîne est vide, pas un nombre
  for(char i = 0; i < str.length(); i++)  // tester tous les caractères, un à la fois
  {
    if (!(isDigit(str.charAt(i)) ))     // Est-ce que le caractère est un chiffre?
    {
       return false;  // Non, retourner Faux
    }
  }
  return true;  // Ici, tous les caractères ont été vérifiés comme étant des chiffres. Retourner Vrai.
}

//------------------------------------------------------------------------------
 // procédure de configuration du STM32, style arduino 

void setup() 
{
  // configurations des broches divers
  pinMode(FQ_UD, OUTPUT);   // Sortie
  pinMode(W_CLK, OUTPUT);   // Sortie
  pinMode(DATA, OUTPUT);    // Sortie
  pinMode(RESET, OUTPUT);   // Sortie
  pinMode(pinLED, OUTPUT);  // Sortie

  Serial.begin(9600);   // configuration du port série (broches A9 et A10)
 
  digitalWrite(pinLED, HIGH); // éteindre la DEL
  pulseHigh(RESET);   // pulsations initiales envoyées au AD9850
  pulseHigh(W_CLK);   // "
  pulseHigh(FQ_UD);   // "  Cette pulsation démarre le mode série
}

//------------------------------------------------------------------------------
 // Boucle principale, style arduino

void loop() 
{
  // Définition des variables
  int freq_word;  // Le mot à envoyer au AD9850. "int" est un mot de 32-bits sur un processeur 32 bits.
  String freq_input = "";  // La chaîne de caractères contenant la fréquence
    
  #define offsetPPM -3.5
        // L'offset du crystal exprimé en PPM.
  int offset = offsetPPM/1000000*125e6;   // L'offset du crystal exprimé en Hz.
  
  Serial.println("Entrez la Frequence (Hz):");  // Afficher à l'écran la demande de fréquence à l'usager
  freq_input = Serial.readStringUntil(LF);    // Lire la chaîne de caractères entrée par l'usager, délimitée par LF (ASCII 10)
  if (IsNumeric(freq_input))    // Si la chaîne de caractères représente un nombre, envoyer le nouveau mot au AD 9850
  {
    freq_word = round(freq_input.toInt() / (125e6-offset) * 4294967296);  // Calcul du mot selon la formule de la datasheet. 4294967295 c'est (2ᶺ32)  // 
   Serial.print("   Mot de 32 bits non-arrondi:  ");
   Serial.println(freq_input.toInt() / (125e6-offset) * 4294967296);
    sendFrequency(freq_word);  // Envoyer la fréquence
    digitalWrite(pinLED, LOW);    // Allumer la DEL 
    delay(1000);     // Delai de 1000ms
    digitalWrite(pinLED, HIGH);     // Éteindre la DEL.
    freq_input = "";      // Ré-initialiser la chaîne de caractères à vide.
  }
}

